"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createReadModelsReducer;
exports.dropKey = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _assign = _interopRequireDefault(require("@babel/runtime/core-js/object/assign"));

var _getPrototypeOf = _interopRequireDefault(require("@babel/runtime/core-js/object/get-prototype-of"));

var _create = _interopRequireDefault(require("@babel/runtime/core-js/object/create"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _objectSpread16 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _diffJson = require("diff-json");

var _get_hash = _interopRequireDefault(require("./get_hash"));

var _action_types = require("./action_types");

var _constants = require("./constants");

var dropKey = function dropKey(state, key) {
  var nextState = (0, _objectSpread16.default)({}, state);
  delete nextState[key];
  return nextState;
};

exports.dropKey = dropKey;

var refreshUpdatedObjects = function refreshUpdatedObjects(updatedObject, changes) {
  var embeddedKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '$index';
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    var _loop = function _loop() {
      var _step$value = _step.value,
          key = _step$value.key,
          nextChanges = _step$value.changes,
          nextEmbeddedKey = _step$value.embeddedKey;
      var calcKey = embeddedKey !== '$index' && Array.isArray(updatedObject) ? updatedObject.reduce(function (result, value, idx) {
        return value[embeddedKey] === key ? idx : result;
      }, 0) : key;

      if (updatedObject[calcKey] == null || updatedObject[calcKey].constructor === String || updatedObject[calcKey].constructor === Number || updatedObject[calcKey].constructor === Date) {
        return "continue";
      }

      if (Array.isArray(updatedObject[calcKey])) {
        updatedObject[calcKey] = (0, _toConsumableArray2.default)(updatedObject[calcKey]);
      } else {
        var nextObject = (0, _create.default)((0, _getPrototypeOf.default)(updatedObject[calcKey]));
        (0, _assign.default)(nextObject, updatedObject[calcKey]);
        updatedObject[calcKey] = nextObject;
      }

      if (Array.isArray(nextChanges)) {
        refreshUpdatedObjects(updatedObject[calcKey], nextChanges, nextEmbeddedKey);
      }
    };

    for (var _iterator = (0, _getIterator2.default)(changes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ret = _loop();

      if (_ret === "continue") continue;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
};

function createReadModelsReducer() {
  var _state;

  var handlers = {};

  handlers[_action_types.LOAD_READMODEL_STATE_REQUEST] = function (state, action) {
    var readModelName = action.readModelName;
    var resolverName = (0, _get_hash.default)(action.resolverName);
    var resolverArgs = (0, _get_hash.default)(action.resolverArgs);
    return (0, _objectSpread16.default)({}, state, (0, _defineProperty2.default)({}, _constants.connectorMetaMap, (0, _objectSpread16.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, "".concat(readModelName).concat(resolverName).concat(resolverArgs), {
      isLoading: true,
      isFailure: false
    }))));
  };

  handlers[_action_types.LOAD_READMODEL_STATE_SUCCESS] = function (state, action) {
    var _objectSpread8;

    var readModelName = action.readModelName;
    var resolverName = (0, _get_hash.default)(action.resolverName);
    var resolverArgs = (0, _get_hash.default)(action.resolverArgs);
    var readModelState = action.result;
    var key = "".concat(readModelName).concat(resolverName).concat(resolverArgs);
    return (0, _objectSpread16.default)({}, state, (_objectSpread8 = {}, (0, _defineProperty2.default)(_objectSpread8, readModelName, (0, _objectSpread16.default)({}, state[readModelName] || {}, (0, _defineProperty2.default)({}, resolverName, (0, _objectSpread16.default)({}, (state[readModelName] || {})[resolverName] || {}, (0, _defineProperty2.default)({}, resolverArgs, readModelState))))), (0, _defineProperty2.default)(_objectSpread8, _constants.connectorMetaMap, (0, _objectSpread16.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, key, {
      isLoading: false,
      isFailure: false
    }))), (0, _defineProperty2.default)(_objectSpread8, _constants.diffVersionsMap, (0, _objectSpread16.default)({}, state[_constants.diffVersionsMap], (0, _defineProperty2.default)({}, key, 0))), _objectSpread8));
  };

  handlers[_action_types.LOAD_READMODEL_STATE_FAILURE] = function (state, action) {
    var readModelName = action.readModelName;
    var resolverName = (0, _get_hash.default)(action.resolverName);
    var resolverArgs = (0, _get_hash.default)(action.resolverArgs);
    var error = action.error;
    var key = "".concat(readModelName).concat(resolverName).concat(resolverArgs);
    return (0, _objectSpread16.default)({}, state, (0, _defineProperty2.default)({}, _constants.connectorMetaMap, (0, _objectSpread16.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, key, {
      isLoading: false,
      isFailure: true,
      error: error
    }))));
  };

  handlers[_action_types.DROP_READMODEL_STATE] = function (state, action) {
    var _objectSpread12;

    var readModelName = action.readModelName;
    var resolverName = (0, _get_hash.default)(action.resolverName);
    var resolverArgs = (0, _get_hash.default)(action.resolverArgs);
    var key = "".concat(readModelName).concat(resolverName).concat(resolverArgs);
    return (0, _objectSpread16.default)({}, state, (_objectSpread12 = {}, (0, _defineProperty2.default)(_objectSpread12, readModelName, (0, _objectSpread16.default)({}, state[readModelName], (0, _defineProperty2.default)({}, resolverName, dropKey(state[readModelName][resolverName], resolverArgs)))), (0, _defineProperty2.default)(_objectSpread12, _constants.connectorMetaMap, dropKey(state[_constants.connectorMetaMap], key)), (0, _defineProperty2.default)(_objectSpread12, _constants.diffVersionsMap, dropKey(state[_constants.diffVersionsMap], key)), _objectSpread12));
  };

  handlers[_action_types.APPLY_READMODEL_DIFF] = function (state, action) {
    var readModelName = action.readModelName;
    var resolverName = (0, _get_hash.default)(action.resolverName);
    var resolverArgs = (0, _get_hash.default)(action.resolverArgs);
    var wrappedReadModelState = {
      wrap: state[readModelName][resolverName][resolverArgs]
    };
    (0, _diffJson.applyChanges)(wrappedReadModelState, action.diff);
    refreshUpdatedObjects(wrappedReadModelState, action.diff);
    return (0, _objectSpread16.default)({}, state, (0, _defineProperty2.default)({}, readModelName, (0, _objectSpread16.default)({}, state[readModelName], (0, _defineProperty2.default)({}, resolverName, (0, _objectSpread16.default)({}, state[readModelName][resolverName], (0, _defineProperty2.default)({}, resolverArgs, wrappedReadModelState.wrap))))));
  };

  var state = (_state = {}, (0, _defineProperty2.default)(_state, _constants.connectorMetaMap, {}), (0, _defineProperty2.default)(_state, _constants.diffVersionsMap, {}), _state);
  return function (_, action) {
    var eventHandler = handlers[action.type];

    if (eventHandler) {
      state = eventHandler(state, action);
    }

    return state;
  };
}
//# sourceMappingURL=create_read_models_reducer.js.map