"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createViewModelsReducer;
exports.dropKey = void 0;

var _set = _interopRequireDefault(require("@babel/runtime/core-js/set"));

var _from = _interopRequireDefault(require("@babel/runtime/core-js/array/from"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _map = _interopRequireDefault(require("@babel/runtime/core-js/map"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread13 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _get_hash = _interopRequireDefault(require("./get_hash"));

var _action_types = require("./action_types");

var _constants = require("./constants");

var dropKey = function dropKey(state, key) {
  var nextState = (0, _objectSpread13.default)({}, state);
  delete nextState[key];
  return nextState;
};

exports.dropKey = dropKey;

function createViewModelsReducer(viewModels) {
  var _state;

  var handlers = {};

  handlers[_action_types.LOAD_VIEWMODEL_STATE_REQUEST] = function (state, action) {
    var viewModelName = action.viewModelName;
    var aggregateIds = (0, _get_hash.default)(action.aggregateIds);
    var aggregateArgs = (0, _get_hash.default)(action.aggregateArgs);
    return (0, _objectSpread13.default)({}, state, (0, _defineProperty2.default)({}, _constants.connectorMetaMap, (0, _objectSpread13.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, "".concat(viewModelName).concat(aggregateIds).concat(aggregateArgs), {
      isLoading: true,
      isFailure: false
    }))));
  };

  handlers[_action_types.LOAD_VIEWMODEL_STATE_SUCCESS] = function (state, action) {
    var _objectSpread8;

    var viewModelName = action.viewModelName;
    var aggregateIds = (0, _get_hash.default)(action.aggregateIds);
    var aggregateArgs = (0, _get_hash.default)(action.aggregateArgs);
    var viewModelState = action.state;
    var viewModelAggregateVersionsMap = action.aggregateVersionsMap;
    var key = "".concat(viewModelName).concat(aggregateIds).concat(aggregateArgs);
    return (0, _objectSpread13.default)({}, state, (_objectSpread8 = {}, (0, _defineProperty2.default)(_objectSpread8, viewModelName, (0, _objectSpread13.default)({}, state[viewModelName] || {}, (0, _defineProperty2.default)({}, aggregateIds, (0, _objectSpread13.default)({}, (state[viewModelName] || {})[aggregateIds] || {}, (0, _defineProperty2.default)({}, aggregateArgs, viewModelState))))), (0, _defineProperty2.default)(_objectSpread8, _constants.connectorMetaMap, (0, _objectSpread13.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, key, {
      isLoading: false,
      isFailure: false
    }))), (0, _defineProperty2.default)(_objectSpread8, _constants.aggregateVersionsMap, (0, _objectSpread13.default)({}, state[_constants.aggregateVersionsMap], (0, _defineProperty2.default)({}, key, viewModelAggregateVersionsMap))), _objectSpread8));
  };

  handlers[_action_types.LOAD_VIEWMODEL_STATE_FAILURE] = function (state, action) {
    var viewModelName = action.viewModelName;
    var aggregateIds = (0, _get_hash.default)(action.aggregateIds);
    var aggregateArgs = (0, _get_hash.default)(action.aggregateArgs);
    var error = action.error;
    var key = "".concat(viewModelName).concat(aggregateIds).concat(aggregateArgs);
    return (0, _objectSpread13.default)({}, state, (0, _defineProperty2.default)({}, _constants.connectorMetaMap, (0, _objectSpread13.default)({}, state[_constants.connectorMetaMap], (0, _defineProperty2.default)({}, key, {
      isLoading: false,
      isFailure: true,
      error: error
    }))));
  };

  handlers[_action_types.DROP_VIEWMODEL_STATE] = function (state, action) {
    var _objectSpread12;

    var viewModelName = action.viewModelName;
    var aggregateIds = (0, _get_hash.default)(action.aggregateIds);
    var aggregateArgs = (0, _get_hash.default)(action.aggregateArgs);
    var key = "".concat(viewModelName).concat(aggregateIds).concat(aggregateArgs);
    return (0, _objectSpread13.default)({}, state, (_objectSpread12 = {}, (0, _defineProperty2.default)(_objectSpread12, viewModelName, (0, _objectSpread13.default)({}, state[viewModelName], (0, _defineProperty2.default)({}, aggregateIds, dropKey(state[viewModelName][aggregateIds], aggregateArgs)))), (0, _defineProperty2.default)(_objectSpread12, _constants.connectorMetaMap, dropKey(state[_constants.connectorMetaMap], key)), (0, _defineProperty2.default)(_objectSpread12, _constants.aggregateVersionsMap, dropKey(state[_constants.aggregateVersionsMap], key)), _objectSpread12));
  };

  var aggregateHash = new _map.default();

  handlers[_action_types.CONNECT_VIEWMODEL] = function (state, action) {
    if (action.aggregateIds !== '*') {
      var aggregatesKey = (0, _get_hash.default)(action.aggregateIds);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator2.default)(action.aggregateIds), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var aggregateId = _step.value;

          if (!aggregateHash.has(aggregateId)) {
            aggregateHash.set(aggregateId, []);
          }

          aggregateHash.get(aggregateId).push(aggregatesKey);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    return state;
  };

  handlers[_action_types.DISCONNECT_VIEWMODEL] = function (state, action) {
    if (action.aggregateIds !== '*') {
      var aggregatesKey = (0, _get_hash.default)(action.aggregateIds);
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator2.default)(action.aggregateIds), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var aggregateId = _step2.value;
          var aggregateKeys = aggregateHash.get(aggregateId);
          var idx = aggregateKeys.indexOf(aggregatesKey);

          if (idx >= 0) {
            aggregateKeys.splice(idx, 1);
          }

          if (aggregateKeys.length === 0) {
            aggregateHash.delete(aggregateId);
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }

    return state;
  };

  var uniqueListeners = new _map.default();
  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = (0, _getIterator2.default)(viewModels), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var viewModel = _step3.value;
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = (0, _getIterator2.default)((0, _keys.default)(viewModel.projection).filter(function (eventType) {
          return eventType !== 'Init';
        })), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var eventType = _step5.value;

          if (!uniqueListeners.has(eventType)) {
            uniqueListeners.set(eventType, []);
          }

          uniqueListeners.get(eventType).push(viewModel);
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return != null) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  var _iteratorNormalCompletion4 = true;
  var _didIteratorError4 = false;
  var _iteratorError4 = undefined;

  try {
    var _loop = function _loop() {
      var _step4$value = (0, _slicedToArray2.default)(_step4.value, 2),
          eventType = _step4$value[0],
          viewModels = _step4$value[1];

      handlers[eventType] = function (state, action) {
        var _iteratorNormalCompletion6 = true;
        var _didIteratorError6 = false;
        var _iteratorError6 = undefined;

        try {
          for (var _iterator6 = (0, _getIterator2.default)(viewModels), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
            var _viewModel = _step6.value;
            var handler = _viewModel.projection[action.type];
            var aggregateKeys = (0, _from.default)(new _set.default(aggregateHash.get(action.aggregateId)));

            if (aggregateKeys) {
              var _iteratorNormalCompletion7 = true;
              var _didIteratorError7 = false;
              var _iteratorError7 = undefined;

              try {
                for (var _iterator7 = (0, _getIterator2.default)(aggregateKeys), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                  var aggregateKey = _step7.value;

                  var _arr = (0, _keys.default)(state[_viewModel.name][aggregateKey]);

                  for (var _i = 0; _i < _arr.length; _i++) {
                    var aggregateArgs = _arr[_i];
                    state[_viewModel.name][aggregateKey][aggregateArgs] = handler(state[_viewModel.name][aggregateKey][aggregateArgs], action);
                  }

                  state[_viewModel.name][aggregateKey] = (0, _objectSpread13.default)({}, state[_viewModel.name][aggregateKey]);
                }
              } catch (err) {
                _didIteratorError7 = true;
                _iteratorError7 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion7 && _iterator7.return != null) {
                    _iterator7.return();
                  }
                } finally {
                  if (_didIteratorError7) {
                    throw _iteratorError7;
                  }
                }
              }
            }

            if (state[_viewModel.name] && state[_viewModel.name]['*']) {
              var _arr2 = (0, _keys.default)(state[_viewModel.name]['*']);

              for (var _i2 = 0; _i2 < _arr2.length; _i2++) {
                var _aggregateArgs = _arr2[_i2];
                state[_viewModel.name]['*'][_aggregateArgs] = handler(state[_viewModel.name]['*'][_aggregateArgs], action);
              }

              state[_viewModel.name]['*'] = (0, _objectSpread13.default)({}, state[_viewModel.name]['*']);
            }

            state[_viewModel.name] = (0, _objectSpread13.default)({}, state[_viewModel.name]);
          }
        } catch (err) {
          _didIteratorError6 = true;
          _iteratorError6 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion6 && _iterator6.return != null) {
              _iterator6.return();
            }
          } finally {
            if (_didIteratorError6) {
              throw _iteratorError6;
            }
          }
        }

        return (0, _objectSpread13.default)({}, state);
      };
    };

    for (var _iterator4 = (0, _getIterator2.default)(uniqueListeners.entries()), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
      _loop();
    }
  } catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion4 && _iterator4.return != null) {
        _iterator4.return();
      }
    } finally {
      if (_didIteratorError4) {
        throw _iteratorError4;
      }
    }
  }

  var state = (_state = {}, (0, _defineProperty2.default)(_state, _constants.connectorMetaMap, {}), (0, _defineProperty2.default)(_state, _constants.aggregateVersionsMap, {}), _state);
  return function (_, action) {
    var eventHandler = handlers[action.type];

    if (eventHandler) {
      state = eventHandler(state, action);
    }

    return state;
  };
}
//# sourceMappingURL=create_view_models_reducer.js.map